package com.hai.service.impl;

import com.hai.common.utils.DateUtil;
import com.hai.common.utils.IDGenerator;
import com.hai.dao.HighActivityAwardActualPrizeMapper;
import com.hai.dao.HighActivityAwardMapper;
import com.hai.dao.HighActivityAwardSerialNumMapper;
import com.hai.dao.HighActivityPrizeMapper;
import com.hai.entity.*;
import com.hai.enum_type.ActivityAwardStatus;
import com.hai.enum_type.ActivityInfoStatus;
import com.hai.service.HighActivityAwardService;
import com.hai.service.HighDiscountAgentCodeService;
import com.hai.service.HighDiscountAgentRelService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Service("activityAwardService")
public class HighActivityAwardServiceImpl implements HighActivityAwardService {

    @Resource
    private HighActivityAwardMapper highActivityAwardMapper;

    @Resource
    private HighActivityPrizeMapper highActivityPrizeMapper;

    @Resource
    private HighActivityAwardSerialNumMapper highActivityAwardSerialNumM;

    @Resource
    private HighDiscountAgentRelService highDiscountAgentRelService;

    @Resource
    private HighDiscountAgentCodeService highDiscountAgentCodeService;


    @Override
    public List<HighActivityAward> getListByActivityId(Long activityId) {
        HighActivityAwardExample example = new HighActivityAwardExample();
        example.createCriteria().andActivityInfoIdEqualTo(activityId).andStatusNotEqualTo(0);
        return highActivityAwardMapper.selectByExample(example);
    }

    @Override
    @Transactional(rollbackFor=Exception.class,propagation= Propagation.REQUIRES_NEW)
    public void editHighActivateAward(HighActivityAward highActivityAward, List<HighActivityPrize> highActivityPrizes)  throws Exception  {

        // 增加奖励
        if (highActivityAward.getId() == null) {
            highActivityAward.setCreateTime(new Date());
            highActivityAward.setUpdateTime(new Date());
            highActivityAward.setStatus(ActivityAwardStatus.status1.getNumber());
            highActivityAwardMapper.insert(highActivityAward);
        } else {
            highActivityAward.setUpdateTime(new Date());
            highActivityAwardMapper.updateByPrimaryKey(highActivityAward);
        }

        HighActivityAwardSerialNum serialNum;

        HighActivityAwardActualPrize actualPrize;

        for (int i = 0; i < highActivityAward.getNum(); i++) {
            serialNum = new HighActivityAwardSerialNum();
            serialNum.setActivityInfoId(highActivityAward.getActivityInfoId());
            serialNum.setActivityAwardId(highActivityAward.getId());
            serialNum.setSerialNum("HFA" + DateUtil.date2String(new Date(),"yyyyMMddHHmmss") + IDGenerator.nextId(5));
            serialNum.setStatus(1);
            serialNum.setCreateTime(new Date());
            highActivityAwardSerialNumM.insert(serialNum);

            // 增加奖品
            for (HighActivityPrize list : highActivityPrizes) {
                list.setActivityAwardId(highActivityAward.getId());
                list.setCreateTime(new Date());
                list.setStatus(1);
                highActivityPrizeMapper.insert(list);
                HighDiscountAgentRel discountAgentRel = highDiscountAgentRelService.getRelByDiscountAgent(list.getPrizeId() , Long.valueOf(list.getExt1()));

                List<HighDiscountAgentCode> codeList = highDiscountAgentCodeService.getDiscountCodeByDiscountAgentId(discountAgentRel.getId() , list.getPrizeNum());

                for (HighDiscountAgentCode agentCode:  codeList) {
                    actualPrize = new HighActivityAwardActualPrize();
                    actualPrize.setActivityAwardPrizeId(list.getId());
                    actualPrize.setActivityAwardSerialNumId(serialNum.getId());
                    actualPrize.setActualPrizeType(2);
                    actualPrize.setActualPrizeId(agentCode.getId());
                    actualPrize.setStatus(1);
                    actualPrize.setCreateTime(new Date());
                }

            }

        }



    }

    @Override
    public HighActivityAward getDetailById(Long id) {
        return highActivityAwardMapper.selectByPrimaryKey(id);
    }
}
